
/* $Id: ea4cedef91e65798d6187d601be284ab669f41b6 $
   rotation.c: rotate voh type vobs.
   Copyright (C) 2012 J. A. Green <green8@sdf-eu.org>, If copyright allows, please addend copyright to additions to this source.
   lic: GNU GPL v3, see COPYING
*/

#include <math.h>
#include "progscope.h" /* !! */

/* given a 3d vector orgin be that place where it would have 0 magnitude,
   rotate its direction thru its xy plane about its orgin by ang_dpl radians */
void rxy_vrt_vf(vf_t * pv, float ang_dpl) {
	/* needs better method. will be calling with v_axi */
	float planar_radius, scale, dx, dy, theta;
	/* get planar radius(2d normalize) for deriving scale */
	planar_radius = sqrt(pv->x * pv->x + pv->y * pv->y);
	scale = 1 / planar_radius;
	dx = pv->x * scale;
	dy = pv->y * scale;
	/* determine new angle from displacement and vector vs quadrant sign */
	theta =	ang_dpl + \
		((dy >= 0) \
			? ((dx >= 0) ? asinf(dy) : asinf(-dx) + M_PI_2) \
			: ((dx >= 0) ? asinf(dy) : asinf(dx) - M_PI_2));
	pv->x = planar_radius * cosf(theta);
	pv->y = planar_radius * sinf(theta);
}
/* rotate in zx plane about orgin by ang_dpl radians */
void rzx_gl_vf(vf_t * pv, float ang_dpl) {
	float planar_radius, scale, dz, dx, theta;
	planar_radius = sqrt(pv->z * pv->z + pv->x * pv->x);
	scale = 1 / planar_radius;
	dz = pv->z * scale;
	dx = pv->x * scale;
	/* gl quadrant layout
   	   subtracting 90 degrees z becomes inverted, quadrants are adjusted */
	theta = ang_dpl + \
		((dx >= 0) \
			? ((dz >= 0) ? asinf(dx) - M_PI_2 : asinf(-dz)) \
			: ((dz >= 0) ? asinf(dx) - M_PI_2 : asinf(dz) - M_PI));
	/* adjust sine and cosine for adjusted quadrants */
	pv->z = planar_radius * sinf(theta);
	pv->x = planar_radius * cosf(theta);
}
/* rotate yz plane about orgin by ang_dpl radians */
void ryz_gl_vf(vf_t * pv, float ang_dpl) {
	float planar_radius, scale, dy, dz, theta;
	planar_radius = sqrt(pv->y * pv->y + pv->z * pv->z);
	scale = 1 / planar_radius;
	dy = pv->y * scale;
	dz = pv->z * scale;
	/* gl quadrant layout
   	   adding 90 degrees z becomes inverted and quadrants are adjusted */
	theta =	ang_dpl + \
		((dz >= 0) \
			? ((dy >= 0) ? asinf(dz) + M_PI_2 : asinf(-dy) + M_PI) \
			: ((dy >= 0) ? asinf(dz) + M_PI_2 : asinf(dy)));
	/* adjust sine and cosine for adjusted quadrants */
	pv->y = planar_radius * sinf(theta);
	pv->z = planar_radius * cosf(theta);
}
/* convert un/signed degrees to radian representation */
float deg_to_radf(float deg) {
	return(deg * 0.0174532925199);
}
